<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
namespace FiloBlu\Esb\Plugin\OrderPlace;
use \Magento\Sales\Api\Data\OrderInterface;
use \FiloBlu\Esb\Helper\Data;

/**
 * Plugin that extends OrderManagement. Sends data on order place.
 */
 
 class Plugin{
	
    /**
     * @var \FiloBlu\Esb\Model\ScheduleFactory
     */  	
    protected $_scheduleFactory;
    
	/**
     * @var \Psr\Log\LoggerInterface $logger
     */  
	protected $_logger;

	/**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

	/**
     * @var \FiloBlu\Esb\Helper\Data
     */  
	protected $_helper;
	
	/**
     * @param \FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \FiloBlu\Esb\Helper\Data $helper
     */  
	public function __construct(
		\FiloBlu\Esb\Model\ScheduleFactory $scheduleFactory, 
		\Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \FiloBlu\Esb\Helper\Data $helper
	){
		$this->_scheduleFactory=$scheduleFactory;
		$this->_logger = $logger;
        $this->_scopeConfig = $scopeConfig;
        $this->_helper = $helper;
	}

    /**
     * Send data to ESB schedule after order place. 
     *
     * @param \Magento\Sales\Api\OrderManagementInterface $interceptor
     * @param \Magento\Sales\Api\Data\OrderInterface  $order
     * @return \Magento\Sales\Api\Data\OrderInterface 
     */
    public function afterPlace($interceptor, $order){    
		//$this->_logger->info(json_encode($order->getData()));
		//$this->_logger->info("ORDER ID: ".$order->getId());
        $schedule = $this->_scheduleFactory->create();
               $events=unserialize($this->_scopeConfig->getValue(
            'esb/endpoints/events', 
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        ));  

        foreach($events as $eventData){
            if($eventData["event_type"]!="order_place"){continue;}
            $endpoint=$this->_helper->getEndpointUrl($eventData["event_endpoint"]);
            $schedule->setData(
                array(
                    "object_repository_interface" => "\Magento\Sales\Api\OrderRepositoryInterface",
                    "object_id" => $order->getId(),
                    "endpoint" => $endpoint,
                    "event" => "sales_order_place_after",
                    "status" => "pending",
                    "executable_after" => new \Zend_Db_Expr('NOW()')
                )
            );
            $schedule->save();    
        }    
        return $order;
    }
}