<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
  
namespace FiloBlu\Esb\Setup;
 
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Ddl\Table;
 
class UpgradeSchema implements UpgradeSchemaInterface
{
    protected $_statusFactory;
    public function __construct(
        \Magento\Sales\Model\Order\StatusFactory $statusFactory
    ){
        $this->_statusFactory = $statusFactory;
    }
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();
        
        if (version_compare($context->getVersion(), '1.0.1') < 0) {

            // Get module table
            $tableName = $installer->getTable('esb_schedule');
            $connection = $installer->getConnection();
            if ($setup->getConnection()->isTableExists($tableName) == true) {
                // Declare data
                $connection->addColumn( 
                    $tableName,
                    'object_repository_interface', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'nullable' => true,
                        'comment' => 'Object Repository Interface',
                        'after' => 'object_class'
                    ]
                );
                $connection->modifyColumn(
                    $tableName,
                    'object_class',                    
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'comment' => 'Object Class',
                        'nullable' => true
                    ]
                );
            }
        }
        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $confirmedStatus = $this->_statusFactory->create();
            $confirmedStatus->setData('status', 'confirmed');
            $confirmedStatus->setData('label', 'Confirmed');
            $confirmedStatus->save();            
            $confirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);

            $orderExportedStatus = $this->_statusFactory->create();
            $orderExportedStatus->setData('status', 'order_exported');
            $orderExportedStatus->setData('label', 'Order Exported');
            $orderExportedStatus->save();
            $orderExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);

            $paymentConfirmedStatus = $this->_statusFactory->create();
            $paymentConfirmedStatus->setData('status', 'payment_confirmed');
            $paymentConfirmedStatus->setData('label', 'Payment Confirmed');
            $paymentConfirmedStatus->save();
            $paymentConfirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);   

            $paymentExportedStatus = $this->_statusFactory->create();
            $paymentExportedStatus->setData('status', 'payment_exported');
            $paymentExportedStatus->setData('label', 'Payment Exported');
            $paymentExportedStatus->save();
            $paymentExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);                     

            $invoicedStatus = $this->_statusFactory->create();
            $invoicedStatus->setData('status', 'invoiced');
            $invoicedStatus->setData('label', 'Invoiced');
            $invoicedStatus->save();            
            $invoicedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);

            $shippedStatus = $this->_statusFactory->create();
            $shippedStatus->setData('status', 'shipped');
            $shippedStatus->setData('label', 'Shipped');
            $shippedStatus->save();            
            $shippedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);            
            
        }         
        $installer->endSetup();
    }
}